import click
from loguru import logger

import komodo_cli.printing as printing
from komodo_cli.backends.backend import Backend
from komodo_cli.backends.backend_factory import BackendFactory
from komodo_cli.types import (
    JobNotFoundException,
    JobNotInDesiredStateException,
    JobStatus,
)
from komodo_cli.utils import APIClient, handle_errors, update_context_with_api_client


@click.command("cancel")
@click.argument(
    "job_id",
    type=str,
)
@click.pass_context
@handle_errors
def cmd_cancel(ctx: click.Context, job_id: str):
    """Cancel a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Cancelling job {job_id}")

    api_client: APIClient = ctx.obj["api_client"]
    job = api_client.get_job(job_id)

    backend_schema = api_client.get_backend(job.backend_name)
    backend: Backend = BackendFactory.get_backend(
        backend_schema,
        api_client,
    )

    printing.info(f"Cancelling job {job_id}", bold=True)
    api_client.cancel_job(job_id)
    # only the local backend actually does anything here
    backend.cancel(job.backend_job_id)



import click
from loguru import logger
from tabulate import tabulate

import komodo_cli.printing as printing
from komodo_cli.utils import APIClient, handle_errors, update_context_with_api_client


@click.command("list")
@click.option("--skip", "-s", type=int, default=0)
@click.option("--limit", "-l", type=int, default=10)
@click.pass_context
@handle_errors
def cmd_list(ctx: click.Context, skip: int, limit: int):
    """List Komodo jobs."""
    logger.info("Listing jobs")

    update_context_with_api_client(ctx)
    api_client: APIClient = ctx.obj["api_client"]

    jobs = api_client.list_jobs(skip, limit)

    jobs_to_print = [
        [
            job.id,
            job.command,
            job.status.value,
            job.backend_name,
        ]
        for job in jobs
    ]

    printing.header(f"Found {len(jobs)} Komodo jobs\n", bold=True)
    printing.info(
        tabulate(
            jobs_to_print,
            headers=[
                "Job ID",
                "Command",
                "Status",
                "Backend",
            ],
            tablefmt="simple_grid",
        ),
    )



import os

import click
import yaml

import komodo_cli.printing as printing
from komodo_cli.commands.cmd_run import _run_helper
from komodo_cli.utils import handle_errors


@click.command("do")
@click.option("--file", "-f", type=str, required=True)
@click.option("--detach", "-d", is_flag=True)
@click.pass_context
@handle_errors
def cmd_do(ctx: click.Context, file: str, detach: bool):
    if not os.path.isfile(file):
        printing.error(
            f"{file} does not exist",
            bold=True,
        )

    with open(file, "r") as f:
        task = yaml.load(f, yaml.FullLoader)

    kind = task["kind"]

    if kind == "job":
        # TODO: use vyper for defaults
        backend = task.get("backend", None)
        resource = task.get("resource", None)
        num_nodes = task.get("num_nodes", 1)
        backend_config_override = task.get("config", {})
        command = task["command"]
        _run_helper(
            ctx,
            backend,
            resource,
            num_nodes,
            backend_config_override,
            detach,
            command,
        )
    # TODO: add support for machine tasks
    else:
        printing.error(f"Task of type {kind} is not supported", bold=True)
        exit(1)



import click
from loguru import logger
from tabulate import tabulate

import komodo_cli.printing as printing
from komodo_cli.utils import APIClient, handle_errors, update_context_with_api_client


@click.command("describe")
@click.argument(
    "job_id",
    type=str,
)
@click.pass_context
@handle_errors
def cmd_describe(ctx: click.Context, job_id: str):
    """Describe a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Describing job {job_id}")

    api_client: APIClient = ctx.obj["api_client"]

    job = api_client.get_job(job_id)
    jobs_to_print = [
        [
            job.id,
            job.command,
            job.status.value,
            job.backend_name,
            job.resource_name,
            job.backend_job_id,
        ]
    ]

    click.echo(
        tabulate(
            jobs_to_print,
            headers=[
                "Job ID",
                "Command",
                "Status",
                "Backend",
                "Resource",
                "Backend Job ID",
            ],
            tablefmt="simple_grid",
        ),
    )



import sys

import click
from loguru import logger

import komodo_cli.printing as printing
from komodo_cli.backends.backend import Backend
from komodo_cli.backends.backend_factory import BackendFactory
from komodo_cli.types import JobNotFoundException, JobStatus
from komodo_cli.utils import APIClient, handle_errors, update_context_with_api_client


@click.command("logs")
@click.argument(
    "job_id",
    type=str,
)
@click.option(
    "--node-index",
    "-i",
    type=int,
    default=0,
    help="The index of the node to get logs from",
)
@click.option(
    "--watch",
    "-w",
    type=bool,
    is_flag=True,
    help="Whether to watch the logs or just output current logs.",
)
@click.pass_context
@handle_errors
def cmd_logs(ctx: click.Context, job_id: str, node_index: int, watch: bool):
    """Get logs for a Komodo job."""
    update_context_with_api_client(ctx)
    logger.info(f"Getting logs for job {job_id}, watch={watch}")

    api_client: APIClient = ctx.obj["api_client"]

    job = api_client.get_job(job_id)
    if job.status == JobStatus.PENDING:
        printing.warning(
            f"Job {job_id} is currently pending",
            bold=True,
        )
        return

    if node_index >= job.num_nodes:
        printing.error(
            f"Node index {node_index} is out of range, job {job_id} only has {job.num_nodes} nodes"
        )
        exit(1)

    backend_schema = api_client.get_backend(job.backend_name)
    backend: Backend = BackendFactory.get_backend(
        backend_schema,
        api_client,
    )

    printing.info(f"Getting logs for job {job_id}", bold=True)
    for line in backend.logs(job.backend_job_id, node_index, watch):
        if type(line) != str:
            line = line.decode("utf-8")
        printing.info(f"{line.strip()}")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
This file is used to create the package we'll publish to PyPI.

.. currentmodule:: setup.py
.. moduleauthor:: kmushegi <mushegiani@gmail.com>
"""

import importlib.util
import os
from codecs import open  # Use a consistent encoding.
from os import path
from pathlib import Path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# Get the base version from the library.  (We'll find it in the `version.py`
# file in the src directory, but we'll bypass actually loading up the library.)
vspec = importlib.util.spec_from_file_location(
    "version",
    str(Path(__file__).resolve().parent / "komodo_cli" / "version.py"),
)
vmod = importlib.util.module_from_spec(vspec)
vspec.loader.exec_module(vmod)
version = getattr(vmod, "__version__")

# If the environment has a build number set...
if os.getenv("buildnum") is not None:
    # ...append it to the version.
    version = f"{version}.{os.getenv('buildnum')}"

setup(
    name="komo",
    description="A CLI client for Komodo AI",
    long_description=long_description,
    packages=find_packages(
        exclude=["*.tests", "*.tests.*", "tests.*", "tests"]
    ),
    version=version,
    install_requires=[
        "boto3",
        "click",
        "docker",
        "GitPython",
        "jinja2",
        "kubernetes",
        "loguru",
        "pip-check-reqs",
        "pip-licenses",
        "pylint",
        "pytest",
        "pytest-cov",
        "pytest-pythonpath",
        "requests",
        "setuptools",
        "Sphinx",
        "sphinx-rtd-theme",
        "tabulate",
        "tox",
        "twine",
        "vyper-config",
    ],
    entry_points="""
    [console_scripts]
    komo=komodo_cli.cli:cli
    """,
    python_requires=">=3",
    license=None,  # noqa
    author="Komodo AI",
    author_email="mushegiani@gmail.com",
    # Use the URL to the github repo.
    url="https://github.com/komodoai/komo_cli",
    download_url=(
        f"https://github.com/komodoai/" f"komo_cli/archive/{version}.tar.gz"
    ),
    keywords=[
        "komodo",
        "komodoai",
        "mlops",
        "dev tools",
        "machine learning",
        "infrastructure",
    ],
    # See https://PyPI.python.org/PyPI?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        # "Development Status :: 3 - Beta",
        # Indicate who your project is intended for.
        # "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        # Pick your license.  (It should match "license" above.)
        """License :: OSI Approved :: Apache Software License""",  # noqa
        # noqa
        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
)
